@=# data: {title}
<header style="border-bottom: 1px solid gray; padding-bottom: 2px;">
    <h2>{{: data.get("title") or "ViloLog" :}}</h2>
    <nav class="">
        <a href="/_pages" class="pure-button">Pages</a>
        <a href="/_newPage" class="pure-button">+ New Page</a>
        <a href="/_users" class="pure-button">Users</a>
        <a href="/_newUser" class="pure-button">+ New Users</a>
        <span class="pull-right small">
            <a href="/" target="_blank" class="pure-button">View Blog</a>
            <a href="/_logout" class="pure-button">&gt; Logout</a>
        </span>
    </nav>
</header>
<br>
