@=# data: {blogTitle, blogDescription}
<header style="border-bottom: 1px solid gray;">
    <h2 style="margin-bottom: 0;">
        <a href="/" class="black">{{: data.blogTitle :}}</a>
    </h2>
    <p class="small gray" style="margin-top: 2px;">{{: data.blogDescription :}}</p>
</header>
<br>
