@=# data: {footerLine}
<br>
<footer style="border-top: 1px solid gray;">
    <p class="small gray">{{: data.footerLine :}}</p>
</footer>

<script src="https://cdnjs.cloudflare.com/ajax/libs/highlight.js/10.3.2/highlight.min.js" crossorigin="anonymous"
    integrity="sha512-8DnqlIcZT5O7kn55WxWs0PnFyhwCqVEnsDpyHqcadSNtP0eqae406JmIoo2cXT5A42jnlcCw013sdkmK3mEJcQ=="
></script>
<script>hljs.initHighlightingOnLoad();</script>
