#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['vilolog']

package_data = \
{'': ['*'],
 'vilolog': ['default-admin-theme/*',
             'default-admin-theme/static/*',
             'default-blog-theme/*',
             'default-blog-theme/static/*']}

install_requires = \
['bcrypt >=3.2.0',
 'dotsi >=0.0.3',
 'Markdown >=3.3.3',
 'pogodb >=0.0.2',
 'psycopg2-binary >=2.8.6',
 'python-dateutil >=2.8.1',
 'qree >=0.0.2',
 'vilo >=0.0.4']

setup(name='vilolog',
      version='0.0.5',
      description='ViloLog: Simple blogging engine, built atop Vilo and PogoDB.',
      author='Polydojo, Inc.',
      author_email='8079593+sumukhbarve@users.noreply.github.com',
      url='https://github.com/polydojo/vilolog',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
     )
